@@ -37,7 +37,11 @@ class BaseReader:
         """
         for path in self.get_filepaths():
             yield (
-                xlrd.open_workbook(path, pickleable=0, formatting_info=1),
+                xlrd.open_workbook(
+                    path,
+                    pickleable=0,
+                    formatting_info=1,
+                    on_demand=True),
                 os.path.split(path)[1]
                 )
 
@@ -59,6 +63,8 @@ class BaseReader:
                     filter.row(row_x,row_x)
                     for col_x in xrange(sheet.ncols):
                         filter.cell(row_x,col_x,row_x,col_x)
+                if workbook.on_demand:
+                    workbook.unload_sheet(sheet_x)
         filter.finish()
     
 class BaseFilter:
@@ -228,6 +234,10 @@ class BaseWriter:
         self.wtname = wtbook_name
         self.style_list = []
         self.wtsheet_names = set()
+        # the index of the current wtsheet being written
+        self.wtsheet_index = 0
+        # have we set a visible sheet already?
+        self.sheet_visible = False
         if not rdbook.formatting_info:
             return
         for rdxf in rdbook.xf_list:
@@ -357,13 +367,20 @@ class BaseWriter:
         wtsheet.show_grid = rdsheet.show_grid_lines
         wtsheet.show_headers = rdsheet.show_sheet_headers
         wtsheet.panes_frozen = rdsheet.panes_are_frozen
-        wtsheet.show_empty_as_zero = rdsheet.show_zero_values
+        wtsheet.show_zero_values = rdsheet.show_zero_values
         wtsheet.auto_colour_grid = rdsheet.automatic_grid_line_colour
         wtsheet.cols_right_to_left = rdsheet.columns_from_right_to_left
         wtsheet.show_outline = rdsheet.show_outline_symbols
         wtsheet.remove_splits = rdsheet.remove_splits_if_pane_freeze_is_removed
         wtsheet.selected = rdsheet.sheet_selected
-        wtsheet.sheet_visible = rdsheet.sheet_visible
+        # xlrd doesn't read WINDOW1 records, so we have to make a best
+        # guess at which the active sheet should be:
+        # (at a guess, only one sheet should be marked as visible)
+        if not self.sheet_visible and rdsheet.sheet_visible:
+            self.wtbook.active_sheet = self.wtsheet_index
+            wtsheet.sheet_visible = 1
+        self.wtsheet_index +=1
+        
         wtsheet.page_preview = rdsheet.show_in_page_break_preview
         wtsheet.first_visible_row = rdsheet.first_visible_rowx
         wtsheet.first_visible_col = rdsheet.first_visible_colx
@@ -454,7 +471,7 @@ class BaseWriter:
             wtrow.set_cell_blank(wtcolx, style)
         elif cty == xlrd.XL_CELL_BOOLEAN:
             wtrow.set_cell_boolean(wtcolx, cell.value, style)
-        elif cty == error_type:
+        elif cty == xlrd.XL_CELL_ERROR:
             wtrow.set_cell_error(wtcolx, cell.value, style)
         else:
             raise Exception(
@@ -625,7 +642,19 @@ class ErrorFilter(BaseReader,BaseWriter):
         filenames.sort()
         for i,filename,pathname in filenames:
             yield (
-                xlrd.open_workbook(os.path.join(self.temp_path,pathname),pickleable=0,formatting_info=1),
+                # We currently don't open with on_demand=True here
+                # as error filters should be lastish in the chain
+                # so there's no much win.
+                # However, if we did, getting rid of the temp dirs
+                # becomes a problem as, on Windows, they can't be
+                # deleted until the xlrd.Book object is done with
+                # and we don't know when that might be :-(
+                xlrd.open_workbook(
+                    os.path.join(self.temp_path,pathname),
+                    pickleable=0,
+                    formatting_info=1,
+                    on_demand=False
+                    ),
                 filename
                 )
 
@@ -637,7 +666,7 @@ class ErrorFilter(BaseReader,BaseWriter):
         cell = self.rdsheet.cell(rdrowx,rdcolx)
         if cell.ctype == xlrd.XL_CELL_EMPTY:
             return
-        if cell.ctype in (xlrd.XL_CELL_ERROR, xlrd.XL_CELL_BOOLEAN):
+        if cell.ctype == xlrd.XL_CELL_ERROR:
             logger.error("Cell %s of sheet %r contains a bad value: %s" % (
                         xlrd.cellname(rdrowx, rdcolx),
                         quoted_sheet_name(self.rdsheet.name),
@@ -654,6 +683,20 @@ class ErrorFilter(BaseReader,BaseWriter):
             self(self.next)
         self.start(create=False)
 
+class Range(object):
+    __slots__ = ('rsn','rr','rc','wr','wc','r','c')
+    def __init__(self,rsn,rr,rc,wr,wc):
+        self.rsn = rsn
+        self.rr = rr
+        self.rc = rc
+        self.wr = wr
+        self.wc = wc
+        self.r = self.c = 1
+    def __repr__(self):
+        return '<range:%r:(%i,%i)->(%i,%i)-r:%i,c:%i>' % (
+            self.rsn,self.rr,self.rc,self.wr,self.wc,self.r,self.c
+            )
+
 class ColumnTrimmer(BaseFilter):
 
     def __init__(self,is_junk=None):
@@ -662,7 +705,7 @@ class ColumnTrimmer(BaseFilter):
     def start(self,chain=True):
         self.rdsheet = None
         self.pending_rdsheet = None
-        self.rows = {}
+        self.ranges = []
         self.max_nonjunk = 0
         self.max = 0
         if chain:
@@ -670,22 +713,28 @@ class ColumnTrimmer(BaseFilter):
         
     def flush(self):
         if self.rdsheet is not None:
-            for wtrowx in sorted(self.rows.keys()):
-                for e in self.rows[wtrowx]:
-                    if isinstance(e,tuple):
-                        rdrowx,rdcolx,wtcolx = e
+            rsn = None
+            for ra in self.ranges:
+                if rsn is None:
+                    rsn = ra.rsn
+                elif ra.rsn!=rsn:
+                    self.rdsheet = self.rdbook.sheet_by_name(ra.rsn)
+                    self.next.set_rdsheet(self.rdsheet)
+                    rsn = ra.rsn
+                for r in range(ra.r):
+                    for c in range(ra.c):
+                        wtcolx=ra.wc+c
                         if wtcolx<=self.max_nonjunk:
-                            self.next.cell(rdrowx,rdcolx,wtrowx,wtcolx)
-                    else:
-                        self.next.set_rdsheet(e)
+                            self.next.cell(ra.rr+r,ra.rc+c,ra.wr+r,ra.wc+c)
             if self.max!=self.max_nonjunk:
                 logger.debug("Number of columns trimmed from %d to %d for sheet %r",
                              self.max+1,
                              self.max_nonjunk+1,
                              quoted_sheet_name(self.wtsheet_name))
         self.start(chain=False)
-                
+
     def workbook(self,rdbook,wtbook_name):
+        self.rdbook = rdbook
         self.flush()
         self.next.workbook(rdbook,wtbook_name)
         
@@ -699,18 +748,36 @@ class ColumnTrimmer(BaseFilter):
         self.pending_rdsheet = rdsheet
         self.rdsheet = rdsheet
     
+    def add_range(self,rdrowx,rdcolx,wtrowx,wtcolx):
+        if len(self.ranges)>1:
+            to_collapse = self.ranges[-1]
+            possible = self.ranges[-2]
+            if to_collapse.rc==possible.rc and \
+               to_collapse.c==possible.c and \
+               to_collapse.rr==possible.rr+possible.r:
+                possible.r+=to_collapse.r
+                self.ranges.pop()
+        self.ranges.append(Range(
+                self.rdsheet.name,rdrowx,rdcolx,wtrowx,wtcolx
+                ))
+        
     def cell(self,rdrowx,rdcolx,wtrowx,wtcolx):
         if wtcolx>self.max:
             self.max = wtcolx
         cell = self.rdsheet.cell(rdrowx,rdcolx)
         if wtcolx>self.max_nonjunk and not cells_all_junk((cell,),self.is_junk):
             self.max_nonjunk = wtcolx
-        if wtrowx not in self.rows:
-            self.rows[wtrowx]=[]
-        if self.pending_rdsheet is not None:
-            self.rows[wtrowx].append(self.pending_rdsheet)
+        if not self.ranges:
+            self.add_range(rdrowx,rdcolx,wtrowx,wtcolx)
+        elif self.pending_rdsheet is not None: 
+            self.add_range(rdrowx,rdcolx,wtrowx,wtcolx)
             self.pending_rdsheet = None
-        self.rows[wtrowx].append((rdrowx,rdcolx,wtcolx))
+        else:
+            r = self.ranges[-1]
+            if rdrowx==r.rr and wtrowx==r.wr and rdcolx==r.rc+r.c and wtcolx==r.wc+r.c:
+                r.c+=1
+            else:
+                self.add_range(rdrowx,rdcolx,wtrowx,wtcolx)
                                                           
     def finish(self):
         self.flush()
