@@ -24,13 +24,16 @@ class DummyBook:
     _sheet_visibility = []
     xf_list = []
     datemode = 0
+    on_demand = False
 
     def __init__(self,formatting_info=0):
         self.formatting_info=formatting_info
         self.__sheets = []
+        self.__name2sheet = {}
         
     def add(self,sheet):
         self.__sheets.append(sheet)
+        self.__name2sheet[sheet.name]=sheet
 
     @property
     def nsheets(self):
@@ -38,8 +41,14 @@ class DummyBook:
 
     def sheet_by_index(self,i):
         return self.__sheets[i]
+
+    def sheet_by_name(self,name):
+        return self.__name2sheet[name]
+
+    def unload_sheet(self,i):
+        pass
         
-def make_book(rows):
+def make_book(rows=[]):
     book = DummyBook()
     sheet = make_sheet(rows,book=book)
     return book
